import uuid

from datetime import date

from sqlalchemy import Date, Enum, PickleType, Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column

from ..db.base_class import reg
from .setting_type import SettingType


@reg.mapped_as_dataclass(unsafe_hash=True)
class DbSetting:
    __tablename__ = "settings"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    name: Mapped[str] = mapped_column(Unicode, unique=True, nullable=False)
    data: Mapped[dict] = mapped_column(PickleType)
    setting_type: Mapped[SettingType] = mapped_column(Enum(SettingType), nullable=False)

    valid_from: Mapped[date | None] = mapped_column(Date(), nullable=True)
    valid_till: Mapped[date | None] = mapped_column(Date(), nullable=True)

    def __init__(
        self,
        setting_type: SettingType,
        name: str,
        data: dict,
        valid_from: date | None = None,
        valid_till: date | None = None,
        id_: uuid.UUID | None = None,
    ) -> None:
        if id_ is not None:
            self.id = id_
        self.setting_type = setting_type
        self.name = name
        self.data = data
        self.valid_from = valid_from
        self.valid_till = valid_till
