import datetime
import uuid

from typing import TYPE_CHECKING

from sqlalchemy import DateTime, UniqueConstraint, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.schema import ForeignKey

from ..db.base_class import reg


if TYPE_CHECKING:
    from .client import Client
    from .user import User


@reg.mapped_as_dataclass(unsafe_hash=True)
class LoginHistory:
    __tablename__ = "login_history"
    __table_args__ = (UniqueConstraint("user_id", "client_id", "date"),)
    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    user_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("users.id"), nullable=False)
    client_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("clients.id"),
        nullable=False,
    )
    date: Mapped[datetime.datetime] = mapped_column(DateTime(), nullable=False)

    user: Mapped["User"] = relationship("User", back_populates="login_history")
    client: Mapped["Client"] = relationship("Client", back_populates="login_history")

    def __init__(
        self,
        user_id: uuid.UUID,
        client_id: uuid.UUID,
        date: datetime.datetime | None = None,
        id_: uuid.UUID | None = None,
    ) -> None:
        self.user_id = user_id
        self.client_id = client_id
        self.date = datetime.datetime.utcnow() if date is None else date
        if id_ is not None:
            self.id = id_
