import datetime
import uuid

from sqlalchemy import Date, Uuid, func, text
from sqlalchemy.dialects import postgresql
from sqlalchemy.orm import Mapped, mapped_column

from ..db.base_class import reg


@reg.mapped_as_dataclass(unsafe_hash=True)
class Period:
    __tablename__ = "periods"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    valid_from: Mapped[datetime.date] = mapped_column(Date, nullable=False)
    valid_till: Mapped[datetime.date] = mapped_column(Date, nullable=False)

    __table_args__ = (
        postgresql.ExcludeConstraint(
            (func.daterange(valid_from, valid_till, text("'[]'")), "&&"),
        ),
    )

    def __init__(
        self,
        valid_from: datetime.date,
        valid_till: datetime.date,
        id_: uuid.UUID | None = None,
    ) -> None:
        self.valid_from = valid_from
        self.valid_till = valid_till
        if id_ is not None:
            self.id = id_
