import uuid

from sqlalchemy import Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg
from .role import Role
from .role_permission import RolePermission


@reg.mapped_as_dataclass(unsafe_hash=True)
class Permission:
    __tablename__ = "permissions"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    name: Mapped[str] = mapped_column(Unicode, unique=True, nullable=False)

    roles: Mapped[list[Role]] = relationship(secondary=RolePermission.__table__, back_populates="permissions")  # type: ignore[attr-defined]

    def __init__(self, name: str, id_: uuid.UUID | None = None) -> None:
        self.name = name
        if id_ is not None:
            self.id = id_
