import uuid

from pydantic import BaseModel, ConfigDict, Field

from . import to_camel
from .cost_centre import CostCentreLink


class AccountLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: str | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)


class AccountBase(BaseModel):
    name: str = Field(..., min_length=1)
    is_starred: bool
    is_active: bool
    cost_centre: CostCentreLink
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class AccountIn(AccountBase):
    type_: int
    is_reconcilable: bool
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)


class Account(AccountIn):
    id_: uuid.UUID
    code: int
    is_fixture: bool


class AccountBlank(AccountIn):
    name: str
    is_fixture: bool
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)
