import uuid

from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import to_camel


class PeriodIn(BaseModel):
    valid_from: date
    valid_till: date
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("valid_from", mode="before")
    @classmethod
    def parse_valid_from(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("valid_from")
    def serialize_valid_from(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")

    @field_validator("valid_till", mode="before")
    @classmethod
    def parse_valid_till(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("valid_till")
    def serialize_valid_till(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")


class Period(PeriodIn):
    id_: uuid.UUID
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class PeriodLink(BaseModel):
    id_: uuid.UUID = Field(...)
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
