import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { CostCentreListResolver } from '../cost-centre/cost-centre-list-resolver.service';

import { AccountDetailComponent } from './account-detail/account-detail.component';
import { AccountListComponent } from './account-list/account-list.component';
import { AccountListResolver } from './account-list-resolver.service';
import { AccountResolver } from './account-resolver.service';
import { AccountTypeResolver } from './account-type-resolver.service';

const accountRoutes: Routes = [
  {
    path: '',
    component: AccountListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Accounts',
    },
    resolve: {
      list: () => inject(AccountListResolver).resolve(),
      accountTypes: () => inject(AccountTypeResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: AccountDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Accounts',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(AccountResolver).resolve(route),
      accountTypes: () => inject(AccountTypeResolver).resolve(),
      costCentres: () => inject(CostCentreListResolver).resolve(),
    },
  },
  {
    path: ':id',
    component: AccountDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Accounts',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(AccountResolver).resolve(route),
      accountTypes: () => inject(AccountTypeResolver).resolve(),
      costCentres: () => inject(CostCentreListResolver).resolve(),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(accountRoutes)],
  exports: [RouterModule],
  providers: [AccountListResolver, AccountResolver],
})
export class AccountRoutingModule {}
