import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { ProductGroup } from '../core/product-group';

const url = '/api/product-groups';
const serviceName = 'ProductGroupService';

@Injectable({
  providedIn: 'root',
})
export class ProductGroupService {
  constructor(
    private http: HttpClient,
    private log: ErrorLoggerService,
  ) {}

  get(id: string | null): Observable<ProductGroup> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return this.http
      .get<ProductGroup>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<ProductGroup>;
  }

  list(): Observable<ProductGroup[]> {
    return this.http
      .get<ProductGroup[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<ProductGroup[]>;
  }

  save(productGroup: ProductGroup): Observable<ProductGroup> {
    return this.http
      .post<ProductGroup>(`${url}`, productGroup)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<ProductGroup>;
  }

  update(productGroup: ProductGroup): Observable<ProductGroup> {
    return this.http
      .put<ProductGroup>(`${url}/${productGroup.id}`, productGroup)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<ProductGroup>;
  }

  saveOrUpdate(productGroup: ProductGroup): Observable<ProductGroup> {
    if (!productGroup.id) {
      return this.save(productGroup);
    }
    return this.update(productGroup);
  }

  delete(id: string): Observable<ProductGroup> {
    return this.http
      .delete<ProductGroup>(`${url}/${id}`)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<ProductGroup>;
  }
}
